<?php
/**
 * Frontend display functionality for Product Review Plugin
 *
 * @package ProductReviewPlugin
 * @since 1.0.0
 */

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly.
}

/**
 * Generate HTML for a product review group with caching
 *
 * @param int $group_id The ID of the review group.
 * @return string The generated HTML.
 */
function generate_product_review_group_html($group_id) {
    // Try to get cached version first
    $cache_key = 'product_review_group_' . $group_id;
    $cached_html = wp_cache_get($cache_key);

    // Return cached version if available and not in admin
    if (false !== $cached_html && !is_admin()) {
        return $cached_html;
    }

    global $wpdb;
    
    $group_table = $wpdb->prefix . 'product_review_groups';
    $review_table = $wpdb->prefix . 'product_reviews';

    // Fetch group data
    $group = $wpdb->get_row($wpdb->prepare(
        "SELECT * FROM {$group_table} WHERE id = %d",
        $group_id
    ));

    if (!$group) {
        return esc_html__('Product review group not found.', 'product-review-shortcode');
    }

    // Fetch reviews
    $reviews = $wpdb->get_results($wpdb->prepare(
        "SELECT * FROM {$review_table} WHERE group_id = %d ORDER BY id ASC",
        $group_id
    ));

    if (empty($reviews)) {
        return esc_html__('No reviews found in this group.', 'product-review-shortcode');
    }

    // Get settings
    $options = get_cached_review_options();

    // Generate HTML
    $html = '<div class="product-review-plugin product-review-group-container">';
    foreach ($reviews as $review) {
        $html .= generate_single_review_html($review, $options);
    }
    $html .= '</div>';

    // Cache the result
    wp_cache_set($cache_key, $html, '', 12 * HOUR_IN_SECONDS);

    return $html;
}

/**
 * Get cached review options
 *
 * @return array Review options
 */
function get_cached_review_options() {
    $cache_key = 'product_review_options';
    $options = wp_cache_get($cache_key);

    if (false === $options) {
        $default_settings = array(
            'background_color' => '#ffffff',
            'text_color' => '#333333',
            'button_color' => '#3b82f6',
            'button_text_color' => '#ffffff',
            'category_label_bg_color' => '#e0f2fe',
            'category_label_text_color' => '#0369a1',
            'price_color' => '#059669',
            'rating_color' => '#fbbf24',
            'feature_name_bg_color' => '#f0f0f0',
            'feature_value_bg_color' => '#ffffff',
            'star_fill_color' => '#FF9529',
            'star_empty_color' => '#e5e7eb',
            'currency' => 'USD',
            'heading_summary' => __('Summary', 'product-review-shortcode'),
            'heading_pros' => __('Pros', 'product-review-shortcode'),
            'heading_cons' => __('Cons', 'product-review-shortcode'),
            'heading_description' => __('Description', 'product-review-shortcode'),
            'heading_who_should_buy' => __('Who Should Buy It?', 'product-review-shortcode'),
        );

        $saved_settings = get_option('product_review_settings', array());
        $options = wp_parse_args($saved_settings, $default_settings);

        wp_cache_set($cache_key, $options, '', 12 * HOUR_IN_SECONDS);
    }

    return $options;
}
/**
 * Generate HTML for a single product review
 *
 * @param object $review The review object.
 * @param array  $options Plugin settings.
 * @return string Generated HTML.
 */
function generate_single_review_html($review, $options) {
    // Sanitize review data
    $product_name = esc_html($review->product_name);
    $product_category = esc_html($review->product_category);
    $product_rating = floatval($review->product_rating);
    $product_price = floatval($review->product_price);
    $image_url = esc_url($review->image_url);
    $button_url = esc_url($review->button_url);
    $button_text = esc_html($review->button_text);

    ob_start();
    ?>
    <div class="product-review-container">
        <div class="category-price-container">
            <div class="category-label"><?php echo $product_category; ?></div>
            <div class="price"><?php echo format_price($product_price, $options['currency']); ?></div>
        </div>

        <h3 class="product-name"><?php echo $product_name; ?></h3>

        <div class="rating-image-container">
            <?php echo generate_star_rating($product_rating, $options['rating_color']); ?>
            <?php if ($image_url): ?>
                <div class="product-image">
                    <img data-src="<?php echo $image_url; ?>" 
                         alt="<?php echo esc_attr($product_name); ?>"
                         src="data:image/svg+xml,%3Csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 1 1'%3E%3C/svg%3E"
                         class="lazy-image">
                </div>
            <?php endif; ?>
        </div>

        <?php
        // Features Table
        $features = json_decode($review->product_features, true);
        if (is_array($features) && !empty($features)) {
            echo generate_features_table($features);
        }
        ?>

        <button class="add-to-cart" onclick="window.location.href='<?php echo $button_url; ?>'">
            <?php echo $button_text; ?>
        </button>

        <?php
        // Expandable Sections
        $sections = array(
            $options['heading_summary'] => wp_kses_post($review->product_summary),
            $options['heading_pros'] => maybe_unserialize($review->product_pros),
            $options['heading_cons'] => maybe_unserialize($review->product_cons),
            $options['heading_description'] => wp_kses_post($review->description),
            $options['heading_who_should_buy'] => maybe_unserialize($review->product_who_should_buy)
        );

        foreach ($sections as $title => $content) {
            if (!empty($content)) {
                echo generate_expandable_section($title, $content);
            }
        }
        ?>
    </div>
    <?php
    return ob_get_clean();
}

/**
 * Generate star rating HTML with proper decimals
 *
 * @param float $rating The rating value
 * @return string Generated HTML
 */
function generate_star_rating($rating) {
    $options = get_cached_review_options();
    // Get colors from options - use the star-specific colors instead of rating_color
    $fill_color = $options['star_fill_color'] ?? '#FF9529';
    $empty_color = $options['star_empty_color'] ?? '#e5e7eb';
    
    // Ensure rating is between 0 and 5
    $rating = max(0, min(5, floatval($rating)));
    $full_stars = floor($rating);
    $decimal = $rating - $full_stars;
    
    // Custom mapping for decimal to percentage fill
    $decimal_percentage = 0;
    switch (number_format($decimal, 1)) {
        case '0.1': $decimal_percentage = 20; break;
        case '0.2': $decimal_percentage = 25; break;
        case '0.3': $decimal_percentage = 30; break;
        case '0.4': $decimal_percentage = 40; break;
        case '0.5': $decimal_percentage = 50; break;
        case '0.6': $decimal_percentage = 60; break;
        case '0.7': $decimal_percentage = 70; break;
        case '0.8': $decimal_percentage = 80; break;
        case '0.9': $decimal_percentage = 85; break;
    }
    
    // Add CSS variables for both fill and empty colors
    $html = '<div class="rating" style="--star-fill-color: ' . esc_attr($fill_color) . '; --star-empty-color: ' . esc_attr($empty_color) . ';">';
    $html .= '<span class="rating-value">' . number_format($rating, 1) . '</span>';
    
    // Generate stars
    for ($i = 1; $i <= 5; $i++) {
        if ($i <= $full_stars) {
            $html .= '<span class="star filled">★</span>';
        } elseif ($i == $full_stars + 1 && $decimal > 0) {
            $html .= sprintf(
                '<span class="star partial" style="--fill-percent: %d%%">★</span>',
                $decimal_percentage
            );
        } else {
            $html .= '<span class="star empty">★</span>';
        }
    }
    
    $html .= '</div>';
    return $html;
}

/**
 * Add star color settings fields
 * 
 * @param array $settings Existing settings
 * @return array Modified settings
 */
function add_star_color_settings($settings) {
    $settings['star_colors'] = array(
        'title' => __('Star Colors', 'product-review-shortcode'),
        'fields' => array(
            'star_fill_color' => array(
                'label' => __('Filled Star Color', 'product-review-shortcode'),
                'type' => 'color',
                'default' => '#FF9529'
            ),
            'star_empty_color' => array(
                'label' => __('Empty Star Color', 'product-review-shortcode'),
                'type' => 'color',
                'default' => '#e5e7eb'
            )
        )
    );
    return $settings;
}
add_filter('product_review_settings_fields', 'add_star_color_settings');
/**
 * Format price with currency
 *
 * @param float  $price Price value.
 * @param string $currency Currency code.
 * @return string Formatted price.
 */
function format_price($price, $currency) {
    $currency_symbols = array(
        'USD' => '$', 'EUR' => '€', 'GBP' => '£', 'JPY' => '¥',
        'BDT' => '৳', 'BRL' => 'R$', 'CAD' => 'C$', 'CHF' => 'CHF',
        'CNY' => '¥', 'HKD' => 'HK$', 'INR' => '₹', 'KRW' => '₩',
        'MXN' => 'MX$', 'NZD' => 'NZ$', 'RUB' => '₽', 'SGD' => 'S$',
        'TRY' => '₺', 'TWD' => 'NT$', 'ZAR' => 'R'
    );

    $symbol = isset($currency_symbols[$currency]) ? $currency_symbols[$currency] : $currency . ' ';
    return $symbol . number_format($price, 2);
}

/**
 * Generate features table HTML
 *
 * @param array $features Features array.
 * @return string Generated HTML.
 */
function generate_features_table($features) {
    if (!is_array($features) || empty($features)) {
        return '';
    }

    $html = '<div class="features-table-container"><table class="features-table"><tbody>';

    foreach ($features as $feature) {
        $html .= sprintf(
            '<tr><td class="feature-name">%s</td><td class="feature-value">%s</td></tr>',
            esc_html($feature['name']),
            esc_html($feature['value'])
        );
    }

    $html .= '</tbody></table></div>';
    return $html;
}

/**
 * Generate expandable section HTML
 *
 * @param string $title Section title.
 * @param mixed  $content Section content.
 * @return string Generated HTML.
 */
function generate_expandable_section($title, $content) {
    $leaf_icon = file_get_contents(plugin_dir_path(__FILE__) . 'assets/icons/leaf.svg');
    
    $html = '<div class="expandable-section">';
    $html .= sprintf(
        '<button class="section-toggle" aria-expanded="false">
            <span class="section-icon">%s</span>
            <span class="section-title">%s</span>
            <span class="toggle-icon">›</span>
        </button>',
        $leaf_icon,
        esc_html($title)
    );
    $html .= '<div class="section-content">';
    
    if ($title === __('Pros', 'product-review-shortcode') || $title === __('Cons', 'product-review-shortcode')) {
        // For pros and cons, treat as newline-separated list
        $html .= generate_list_items($title, $content);
    } else {
        $html .= sprintf('<p>%s</p>', wp_kses_post($content));
    }
    
    $html .= '</div></div>';

    return $html;
}

/**
 * Generate list items HTML
 *
 * @param string $title List title.
 * @param array  $items List items.
 * @return string Generated HTML.
 */
function generate_list_items($title, $items) {
    // Convert textarea content to array if it's a string
    if (is_string($items)) {
        // Split by newline and filter out empty lines
        $items = array_filter(explode("\n", $items), 'trim');
    }

    // If still not an array or empty, return as paragraph
    if (!is_array($items) || empty($items)) {
        return sprintf('<p>%s</p>', esc_html($items));
    }

    // Get plugin directory path
    $plugin_dir = plugin_dir_path(__FILE__);
    
    // Define icon paths
    $check_icon_path = $plugin_dir . 'assets/icons/check.svg';
    $close_icon_path = $plugin_dir . 'assets/icons/close.svg';

    $html = '<ul class="review-list">';
    foreach ($items as $item) {
        $item = trim($item); // Remove any extra whitespace
        if (empty($item)) continue; // Skip empty lines

        $icon = '';
        try {
            if (__('Pros', 'product-review-shortcode') === $title && file_exists($check_icon_path)) {
                $icon = '<span class="icon pro">' . file_get_contents($check_icon_path) . '</span>';
            } elseif (__('Cons', 'product-review-shortcode') === $title && file_exists($close_icon_path)) {
                $icon = '<span class="icon con">' . file_get_contents($close_icon_path) . '</span>';
            }
        } catch (Exception $e) {
            if (current_user_can('manage_options')) {
                error_log('Product Review Plugin: Error loading icon - ' . $e->getMessage());
            }
        }
        
        $html .= sprintf(
            '<li class="item">%s%s</li>',
            $icon,
            esc_html($item)
        );
    }
    $html .= '</ul>';
    
    return $html;
}

/**
 * Clear review cache
 *
 * @param int $group_id The ID of the review group.
 */
function clear_review_cache($group_id) {
    wp_cache_delete('product_review_group_' . $group_id);
}

/**
 * Clear options cache
 */
function clear_options_cache() {
    wp_cache_delete('product_review_options');
}

// Add cache clearing hooks
add_action('update_option_product_review_settings', 'clear_options_cache');

/**
 * Generate schema markup for product reviews
 *
 * @param int $group_id The ID of the review group.
 * @return string Generated schema markup.
 */
function generate_product_review_schema($group_id) {
    global $wpdb;
    $review_table = $wpdb->prefix . 'product_reviews';
    
    // Try to get cached schema
    $cache_key = 'product_review_schema_' . $group_id;
    $cached_schema = wp_cache_get($cache_key);
    
    if (false !== $cached_schema && !is_admin()) {
        return $cached_schema;
    }

    $reviews = $wpdb->get_results($wpdb->prepare(
        "SELECT * FROM {$review_table} WHERE group_id = %d",
        $group_id
    ));

    if (empty($reviews)) {
        return '';
    }

    $schema = array();
    $options = get_cached_review_options();

    foreach ($reviews as $review) {
        // Base product schema
        $product_schema = array(
            '@context' => 'https://schema.org/',
            '@type' => 'Product',
            'name' => $review->product_name,
            'description' => wp_strip_all_tags($review->product_summary),
            'review' => array(
                '@type' => 'Review',
                'reviewRating' => array(
                    '@type' => 'Rating',
                    'ratingValue' => number_format($review->product_rating, 1),
                    'bestRating' => '5',
                    'worstRating' => '1'
                ),
                'author' => array(
                    '@type' => 'Organization',
                    'name' => get_bloginfo('name')
                )
            )
        );

        // Add image if exists
        if (!empty($review->image_url)) {
            $product_schema['image'] = $review->image_url;
        }

        // Add offer if price exists
        if (!empty($review->product_price)) {
            $product_schema['offers'] = array(
                '@type' => 'Offer',
                'url' => $review->button_url,
                'priceCurrency' => $options['currency'],
                'price' => number_format($review->product_price, 2, '.', ''),
                'availability' => 'https://schema.org/InStock',
                'priceValidUntil' => date('Y-m-d', strtotime('+1 year'))
            );
        }

        // Add aggregate rating
        $product_schema['aggregateRating'] = array(
            '@type' => 'AggregateRating',
            'ratingValue' => number_format($review->product_rating, 1),
            'reviewCount' => 1,
            'bestRating' => '5',
            'worstRating' => '1'
        );

        // Add features as product attributes
        $features = json_decode($review->product_features, true);
        if (is_array($features) && !empty($features)) {
            $product_schema['additionalProperty'] = array();
            foreach ($features as $feature) {
                $product_schema['additionalProperty'][] = array(
                    '@type' => 'PropertyValue',
                    'name' => $feature['name'],
                    'value' => $feature['value']
                );
            }
        }

        $schema[] = $product_schema;
    }

    $schema_markup = sprintf(
        '<script type="application/ld+json">%s</script>',
        wp_json_encode($schema, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES)
    );

    // Cache the schema for 12 hours
    wp_cache_set($cache_key, $schema_markup, '', 12 * HOUR_IN_SECONDS);

    return $schema_markup;
}

/**
 * Register frontend assets
 */
function register_frontend_assets() {
    wp_enqueue_style(
        'product-review-style',
        plugin_dir_url(__FILE__) . 'css/frontend-style.css',
        array(),
        filemtime(plugin_dir_path(__FILE__) . 'css/frontend-style.css') // Cache busting
    );

    wp_enqueue_script(
        'product-review-script',
        plugin_dir_url(__FILE__) . 'js/frontend-script.js',
        array('jquery'),
        filemtime(plugin_dir_path(__FILE__) . 'js/frontend-script.js'),
        true
    );

    // Add custom CSS based on settings
    $options = get_cached_review_options();
    $custom_css = generate_custom_css($options);
    wp_add_inline_style('product-review-style', $custom_css);
}
add_action('wp_enqueue_scripts', 'register_frontend_assets');

/**
 * Generate custom CSS based on settings
 *
 * @param array $options Plugin settings.
 * @return string Generated CSS.
 */
function generate_custom_css($options) {
    ob_start();
    ?>
    .product-review-plugin .product-review-container {
        background-color: <?php echo esc_html($options['background_color']); ?>;
        color: <?php echo esc_html($options['text_color']); ?>;
        font-size: <?php echo esc_html($options['font_size']); ?>px;
    }

    .product-review-plugin .add-to-cart {
        background-color: <?php echo esc_html($options['button_color']); ?>;
        color: <?php echo esc_html($options['button_text_color']); ?>;
        font-size: <?php echo esc_html($options['button_font_size']); ?>px;
    }

    .product-review-plugin .category-label {
        background-color: <?php echo esc_html($options['category_label_bg_color']); ?>;
        color: <?php echo esc_html($options['category_label_text_color']); ?>;
        font-size: <?php echo esc_html($options['category_label_font_size']); ?>px;
    }

    .product-review-plugin .price {
        font-size: <?php echo esc_html($options['price_font_size']); ?>px;
        color: <?php echo esc_html($options['price_color']); ?>;
    }

    .product-review-plugin .product-name {
        font-size: <?php echo esc_html($options['product_name_font_size']); ?>px;
    }

    .product-review-plugin .rating-value {
        font-size: <?php echo esc_html($options['rating_font_size']); ?>px;
    }

    .product-review-plugin .section-toggle .section-title {
        font-size: <?php echo esc_html($options['section_title_font_size']); ?>px;
    }

    .product-review-plugin .section-content {
        font-size: <?php echo esc_html($options['section_content_font_size']); ?>px;
    }

    .product-review-plugin .features-table {
        font-size: <?php echo esc_html($options['section_content_font_size']); ?>px;
    }

    .product-review-plugin .star {
        font-size: <?php echo esc_html($options['rating_font_size']); ?>px;
    }
    <?php
    return ob_get_clean();
}